@@ -1,3 +1,23 @@
+/* 
+ * Copyright (C) 2001, 2002 Red Hat Inc.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -418,12 +438,24 @@ get_L_libs (Package *pkg)
   return pkg->L_libs;
 }
 
+static GSList*
+get_other_libs (Package *pkg)
+{  
+  return pkg->other_libs;
+}
+
 static GSList *
 get_I_cflags (Package *pkg)
 {
   return pkg->I_cflags;
 }
 
+static GSList *
+get_other_cflags (Package *pkg)
+{
+  return pkg->other_cflags;
+}
+
 static GSList *
 get_conflicts (Package *pkg)
 {
@@ -529,6 +561,7 @@ fill_list_single_package (Package *pkg, GetListFunc func,
   GSList *tmp;
 
   /* Get list of packages */
+  packages = NULL;
   packages = g_slist_append (packages, pkg);
   recursive_fill_list (pkg, get_requires, &packages);
   
@@ -608,16 +641,37 @@ compare_package_keys (gconstpointer a, gconstpointer b)
   return strcmp (pkg_a->key, pkg_b->key);
 }
 
+static GSList *
+add_env_variable_to_list (GSList *list, const gchar *env)
+{
+  gchar **values;
+  gint i;
+
+  /* FIXME: the separator should be a ';' on Windows
+   */
+  values = g_strsplit (env, ":", 0);
+  for (i = 0; values[i] != NULL; i++)
+    {
+      list = g_slist_append (list, g_strdup (values[i]));
+    }
+  g_strfreev (values);
+
+  return NULL;
+}
+
 static void
 verify_package (Package *pkg)
 {
   GSList *requires = NULL;
   GSList *conflicts = NULL;
+  GSList *system_directories = NULL;
   GSList *iter;
   GSList *requires_iter;
-  GSList *conflicts_iter;  
+  GSList *conflicts_iter;
+  GSList *system_dir_iter = NULL;
   int count;
-  
+  gchar *c_include_path;
+
   /* Be sure we have the required fields */
 
   if (pkg->key == NULL)
@@ -722,24 +776,56 @@ verify_package (Package *pkg)
   g_slist_free (requires);
   g_slist_free (conflicts);
 
+  /* We make a list of system directories that gcc expects so we can remove
+   * them.
+   */
+  system_directories = g_slist_append (NULL, g_strdup ("/usr/include"));
+  
+  c_include_path = g_getenv ("C_INCLUDE_PATH");
+  if (c_include_path != NULL)
+    {
+      system_directories = add_env_variable_to_list (system_directories, c_include_path);
+    }
+  
+  c_include_path = g_getenv ("CPLUS_INCLUDE_PATH");
+  if (c_include_path != NULL)
+    {
+      system_directories = add_env_variable_to_list (system_directories, c_include_path);
+    }
+
   count = 0;
   iter = pkg->I_cflags;
   while (iter != NULL)
     {
+      gint offset = 0;
       /* we put things in canonical -I/usr/include (vs. -I /usr/include) format,
        * but if someone changes it later we may as well be robust
        */
-      if (strcmp (iter->data, "-I/usr/include") == 0 ||
-          strcmp (iter->data, "-I /usr/include") == 0)
+      if (((strcmp (iter->data, "-I") == 0) && (offset = 2))||
+          ((strcmp (iter->data, "-I ") == 0) && (offset = 3)))
         {
-          debug_spew ("Package %s has -I/usr/include in Cflags\n",
-                      pkg->name);
-          if (g_getenv ("PKG_CONFIG_ALLOW_SYSTEM_CFLAGS") == NULL)
-            {
-              iter->data = NULL;
-              ++count;
-              debug_spew ("Removing -I/usr/include from cflags for %s\n", pkg->key);              
-            }
+	  if (offset == 0)
+	    {
+	      iter = iter->next;
+	      continue;
+	    }
+
+	  system_dir_iter = system_directories;
+	  while (system_dir_iter != NULL)
+	    {
+	      if (strcmp (system_dir_iter->data, iter->data + offset) == 0)
+		{
+		  debug_spew ("Package %s has %s in Cflags\n",
+			      pkg->name, (gchar *)iter->data);
+		  if (g_getenv ("PKG_CONFIG_ALLOW_SYSTEM_CFLAGS") == NULL)
+		    {
+		      debug_spew ("Removing %s from cflags for %s\n", iter->data, pkg->key);
+		      ++count;
+		      iter->data = NULL;
+		    }
+		}
+	      system_dir_iter = system_dir_iter->next;
+	    }
         }
 
       iter = iter->next;
@@ -751,6 +837,9 @@ verify_package (Package *pkg)
       --count;
     }
 
+  g_slist_foreach (system_directories, (GFunc) g_free, NULL);
+  g_slist_free (system_directories);
+
   count = 0;
   iter = pkg->L_libs;
   while (iter != NULL)
@@ -764,7 +853,7 @@ verify_package (Package *pkg)
             {              
               iter->data = NULL;
               ++count;
-              debug_spew ("Removing -I/usr/lib from libs for %s\n", pkg->key);
+              debug_spew ("Removing -L/usr/lib from libs for %s\n", pkg->key);
             }
         }
 
@@ -897,36 +986,16 @@ packages_get_L_libs (GSList     *pkgs)
 char *
 package_get_other_libs (Package *pkg)
 {
-  return g_strdup (pkg->other_libs);
+  if (pkg->other_libs_merged == NULL)
+    pkg->other_libs_merged = get_merged (pkg, get_other_libs, TRUE);
+
+  return pkg->other_libs_merged;
 }
 
 char *
 packages_get_other_libs (GSList   *pkgs)
 {
-  GSList *tmp;
-  GString *str;
-  char *retval;
-  
-  str = g_string_new ("");
-  
-  tmp = pkgs;
-  while (tmp != NULL)
-    {
-      Package *pkg = tmp->data;
-
-      if (pkg->other_libs)
-        {
-          g_string_append (str, pkg->other_libs);
-          g_string_append (str, " ");
-        }
-
-      tmp = g_slist_next (tmp);
-    }
-
-  retval = str->str;
-  g_string_free (str, FALSE);
-
-  return retval;
+  return get_multi_merged (pkgs, get_other_libs, TRUE);
 }
 
 char *
@@ -984,36 +1053,16 @@ packages_get_I_cflags (GSList     *pkgs)
 char *
 package_get_other_cflags (Package *pkg)
 {
-  return g_strdup (pkg->other_cflags);
+  if (pkg->other_cflags_merged == NULL)
+    pkg->other_cflags_merged = get_merged (pkg, get_other_cflags, TRUE);
+
+  return pkg->other_cflags_merged;
 }
 
 char *
 packages_get_other_cflags (GSList *pkgs)
 {
-  GSList *tmp;
-  GString *str;
-  char *retval;
-  
-  str = g_string_new ("");
-  
-  tmp = pkgs;
-  while (tmp != NULL)
-    {
-      Package *pkg = tmp->data;
-
-      if (pkg->other_cflags)
-        {
-          g_string_append (str, pkg->other_cflags);
-          g_string_append (str, " ");
-        }
-
-      tmp = g_slist_next (tmp);
-    }
-
-  retval = str->str;
-  g_string_free (str, FALSE);
-
-  return retval;
+  return get_multi_merged (pkgs, get_other_cflags, TRUE);
 }
 
 char *
